/* This program tiles with right trominoes an nxn
   board with one square missing, assuming that n
   is a power of 2.                                */

#include <stdio.h>
#include <stdlib.h>
#include <graphics.h>
#include <conio.h>

#define LL 0
#define UL 1
#define UR 2
#define LR 3

void tromino /* function to do tiling */
     ( int x_board,      /* x coordinate of board */
       int y_board,      /* y coordinate of board */
       int x_missing,    /* x coordinate of missing square */
       int y_missing,    /* y coordinate of missing square */
       int board_size,   /* size of board */
       int k,            /* tile n = 2^k */
       int scale,        /* boardsize / 2^k */
       int color );

void draw_tromino( int x, int y, int orient, int size, int color );

main()
{
     int board_size, /* maximum permitted for screen resolution */
         x_missing,  /* x coordinate of missing square */
         y_missing,  /* y coordinate of missing square */
         k,          /* tile n = 2^k board */
         scale,      /* board_size / 2^k, translates virtual
                        coordinates to screen coordinates */
         to_k,       /* 2^k */
         i,
         gdriver = DETECT,
         gmode,
         errorcode;

     for ( ; ; ) {
          printf( "\nEnter k to tile 2^k X 2^k board (0 to quit): " );
          scanf( "%d", &k );
          if ( k <= 0 )
            break;
          /* compute 2^k */
          to_k = 1;
          for ( i = 0; i < k; i++ )
            to_k *= 2;
          printf( "\nEnter coordinates of missing square: " );
          scanf( "%d%d", &x_missing, &y_missing );
          if ( x_missing >= to_k || y_missing >= to_k ) {
            printf( "\nBad coordinates" );
            continue;
          }

          initgraph( &gdriver, &gmode, "\\tc" );

          errorcode = graphresult();
          if ( errorcode != grOk )
          {
            printf( "Graphics error: %s\n", grapherrormsg( errorcode ) );
            exit( EXIT_FAILURE );
          }
          /* compute largest board size that fits screen */
          board_size = 1;
          while ( board_size <= getmaxy() )
            board_size *= 2;
          board_size /= 2;
          if ( to_k > board_size ) {
            printf( "\nk is too big. Hit any key to resume\n" );
            getch();
            closegraph();
            continue;
          }
          scale = board_size / to_k;

          tromino( 0, 0, x_missing * scale,
                   y_missing * scale, board_size, k, scale, RED );
          getch();
          closegraph();
     }

     return EXIT_SUCCESS;
}

void tromino /* function to do tiling */
     ( int x_board,      /* x coordinate of board */
       int y_board,      /* y coordinate of board */
       int x_missing,    /* x coordinate of missing square */
       int y_missing,    /* y coordinate of missing square */
       int board_size,   /* size of board */
       int k,            /* tile n = 2^k */
       int scale,        /* boardsize / 2^k */
       int color )
{
     int half_size = board_size/2, /* size of subboard */
         x_center,  /* x coordinate of center of board */
         y_center,  /* y coordinate of center of board */
         x_upper_left,  /* x coordinate of missing square in upper
                                                     left subboard */
         y_upper_left,  /* y coordinate of missing square in upper
                                                     left subboard */
         x_upper_right, /* x coordinate of missing square in upper
                                                    right subboard */
         y_upper_right, /* y coordinate of missing square in upper
                                                    right subboard */
         x_lower_right, /* x coordinate of missing square in lower
                                                    right subboard */
         y_lower_right, /* y coordinate of missing square in lower
                                                    right subboard */
         x_lower_left,  /* x coordinate of missing square in lower
                                                     left subboard */
         y_lower_left;  /* y coordinate of missing square in lower
                                                     left subboard */

     if ( k == 1 ) {   /* 2x2 board */
          /* find orientation and draw tromino */
          if ( x_board == x_missing )
               /* missing square in left half */
               if ( y_board == y_missing )
                    /* missing square in upper left */
                    draw_tromino( x_board + scale,
                                  y_board + scale, LR,
                                  scale, color );
               else
                    /* missing square in lower left */
                    draw_tromino( x_board + scale,
                                  y_board + scale, UR,
                                  scale, color );
          else
               /* missing square in right half */
               if ( y_board == y_missing )
                    /* missing square in upper right */
                    draw_tromino( x_board + scale,
                                  y_board + scale, LL,
                                  scale, color );
               else
                    /* missing square in lower right */
                    draw_tromino( x_board + scale,
                                  y_board + scale, UL,
                                  scale, color );
          return;
     }

     /* compute x and y coordinates of center of board */
     x_center = x_board + half_size;
     y_center = y_board + half_size;

     /* Find and print orientation of center tromino.
        Also, set x_upper_left, y_upper_left, ... .  */
     if ( x_missing < x_center) {
          /* missing square in left half */
          x_lower_right = x_upper_right = x_center;
          y_lower_right = y_center;
          y_upper_right = y_center - scale;
          if ( y_missing < y_center ) {
               /* missing square in upper left quadrant */
               x_lower_left = x_center - scale;
               y_lower_left = y_center;
               x_upper_left = x_missing;
               y_upper_left = y_missing;
               draw_tromino( x_center, y_center, LR, scale, GREEN );
          }
          else {
               /* missing square in lower left quadrant */
               x_lower_left = x_missing;
               y_lower_left = y_missing;
               x_upper_left = x_center - scale;
               y_upper_left = y_center - scale;
               draw_tromino( x_center, y_center, UR, scale, GREEN );
          }
     }
     else {
          /* missing square in right half */
          x_lower_left = x_upper_left = x_center - scale;
          y_lower_left = y_center;
          y_upper_left = y_center - scale;
          if ( y_missing < y_center ) {
               /* missing square in upper right quadrant */
               x_lower_right = x_center;
               y_lower_right = y_center;
               x_upper_right = x_missing;
               y_upper_right = y_missing;
               draw_tromino( x_center, y_center, LL, scale, GREEN );
          }
          else {
               /* missing square in lower right quadrant */
               x_lower_right = x_missing;
               y_lower_right = y_missing;
               x_upper_right = x_center;
               y_upper_right = y_center - scale;
               draw_tromino( x_center, y_center, UL, scale, GREEN );
          }
     }

     /* tile the four subboards */
     tromino( x_board, y_board + half_size,
          x_lower_left, y_lower_left, half_size, k - 1,
          scale, RED );
     tromino( x_board + half_size, y_board + half_size,
          x_lower_right, y_lower_right, half_size, k - 1,
          scale, BLUE );
     tromino( x_board + half_size, y_board,
          x_upper_right, y_upper_right, half_size, k - 1,
          scale, RED );
     tromino( x_board, y_board,
          x_upper_left, y_upper_left, half_size, k - 1,
          scale, BLUE );
}

void draw_tromino( int x, int y, int orient, int size, int color )
{
   int poly[ 12 ];

   /* store coords of corners of tromino in poly */
   poly[ 0 ] = x;
   poly[ 1 ] = y;
   switch ( orient ) {
   case LL:
      poly[ 2 ] = x + size;
      poly[ 3 ] = y;
      poly[ 4 ] = x + size;
      poly[ 5 ] = y + size;
      poly[ 6 ] = x - size;
      poly[ 7 ] = y + size;
      poly[ 8 ] = x - size;
      poly[ 9 ] = y - size;
      poly[ 10 ] = x;
      poly[ 11 ] = y - size;
      break;
   case UL:
      poly[ 2 ] = x;
      poly[ 3 ] = y + size;
      poly[ 4 ] = x - size;
      poly[ 5 ] = y + size;
      poly[ 6 ] = x - size;
      poly[ 7 ] = y - size;
      poly[ 8 ] = x + size;
      poly[ 9 ] = y - size;
      poly[ 10 ] = x + size;
      poly[ 11 ] = y;
      break;
   case UR:
      poly[ 2 ] = x - size;
      poly[ 3 ] = y;
      poly[ 4 ] = x - size;
      poly[ 5 ] = y - size;
      poly[ 6 ] = x + size;
      poly[ 7 ] = y - size;
      poly[ 8 ] = x + size;
      poly[ 9 ] = y + size;
      poly[ 10 ] = x;
      poly[ 11 ] = y + size;
      break;
   case LR:
      poly[ 2 ] = x;
      poly[ 3 ] = y - size;
      poly[ 4 ] = x + size;
      poly[ 5 ] = y - size;
      poly[ 6 ] = x + size;
      poly[ 7 ] = y + size;
      poly[ 8 ] = x - size;
      poly[ 9 ] = y + size;
      poly[ 10 ] = x - size;
      poly[ 11 ] = y;
      break;
   }

   /* fill solid with specified color */
   setfillstyle( SOLID_FILL, color );

   /* fillpoly draws the polygon specified by the
      6 points in poly and fills it as directed
      by setfillstyle */
   fillpoly( 6, poly );
}
